==> segment-local-functions-0.1.0.tgz: package/src/cli/init.ts <==
#!/usr/bin/env node

import * as i from 'inquirer'
import * as fs from 'fs-extra'
import * as path from 'path'
import { listAllTrackingPlans } from 'segment-typescript-api/cjs/config_api'
import * as Rx from 'rxjs'
import { generate_protocol_definitions } from '../generate_protocol'
import {exec} from 'child_process'

const TEMPLATE_DIR = path.join(__dirname, '..', '..', 'template')
const SAM_DIR = path.join(TEMPLATE_DIR, 'sam')
const JS_DIR = path.join(TEMPLATE_DIR, 'js')
const JS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'js-project')
const TS_DIR = path.join(TEMPLATE_DIR, 'ts')
const TS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'ts-project')

let cwd = process.cwd()

let answers: {
  language: 'typescript' | 'javascript'
  fn_name: string
  fn_type: 'source' | 'destination'
  work_slug: string
  access_token: string
  tp_name: string
} = {
  fn_name: '',
  fn_type: 'source',
  language: 'javascript',
  work_slug: '',
  access_token: '',
  tp_name: ''
}

var prompts = new Rx.Subject();
i.prompt(prompts as any).ui.process.subscribe(answer => {
  let _answer = answer as any
  (answers as any)[_answer.name as keyof typeof answers] = _answer.answer
  // answers = { ...answers, ...answer as Record<keyof typeof answers, any> }

  if (answers.tp_name) {
    prompts.complete()
  } else if (answers.work_slug && answers.access_token) {
    listAllTrackingPlans(answers.access_token, answers.work_slug).then(tps => {
      prompts.next({
        name: 'tp_name',
        message: 'Select tracking plan',
        type: 'list',
        choices: ['none', ...tps.tracking_plans.map(tps => tps.display_name)]
      })
    }).catch(e => {
      console.error('Error fetching tracking plans')
      console.error(e)
      prompts.complete()
    })
  }
}, error => {
  console.error(error)
}, () => {
  let p1 = fs.copyFile(path.join(SAM_DIR, 'package.json'), path.join(cwd, 'package.json')).then(() => {
    exec('npm i', (error, stdout) => {
      if (error) {
        console.error(error)
      }
    })
  })
  let p2 = fs.copyFile(path.join(SAM_DIR, 'template.yaml'), path.join(cwd, 'template.yaml'))
  let p3: Promise<void>
  let p4: Promise<void>
  let p5: Promise<void>
  let p6: Promise<void>

  if (answers.fn_type == 'source') {
    p3 = fs.copyFile(path.join(SAM_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'))
    p5 = fs.copy(path.join(JS_DIR, 'tests', 'src-payload.json'), path.join(cwd, 'src', 'tests', 'src-payload.json'))
    if (answers.language == 'javascript') {
      p4 = fs.copy(path.join(JS_DIR, 'source.js'), path.join(cwd, 'src', 'function.js'))
      p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-src-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'))
    } else {
      p4 = fs.copy(path.join(TS_DIR, 'source.ts'), path.join(cwd, 'src', 'function.ts'))
      p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-src-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'))
    }
  } else {
    p3 = fs.copyFile(path.join(SAM_DIR, 'sam-dest.js'), path.join(cwd, 'sam.js'))
    p5 = fs.copy(path.join(JS_DIR, 'tests', 'dest-payload.json'), path.join(cwd, 'src', 'tests', 'dest-payload.json'))
    if (answers.language == 'javascript') {
      p4 = fs.copy(path.join(JS_DIR, 'destination.js'), path.join(cwd, 'src', 'function.js'))
      p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-dest-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'))
    } else {
      p4 = fs.copy(path.join(TS_DIR, 'destination.ts'), path.join(cwd, 'src', 'function.ts'))
      p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-dest-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'))
    }
  }

  let p7: Promise<void>
  if (answers.language == 'javascript') {
    p7 = fs.copy(JS_PROJECT_DIR, cwd)
  } else {
    p7 = fs.copy(TS_PROJECT_DIR, cwd)
  }

  if (answers.tp_name !== 'none') {
    generate_protocol_definitions({
      access_token: answers.access_token,
      tp_name: answers.tp_name,
      work_slug: answers.work_slug,
      out_file: path.join(cwd, 'src', 'protocol.d.ts')
    })
  }
});

prompts.next({
  name: 'language',
  message: 'Select language',
  type: 'list',
  choices: ['javascript', 'typescript']
})

prompts.next({
  name: 'fn_type',
  message: 'Select function type',
  type: 'list',
  choices: ['source', 'destination']
})

prompts.next({
  name: 'fn_name',
  message: 'Select function name',
  type: 'input'
})

prompts.next({
  name: 'work_slug',
  message: 'Enter workspace slug',
  type: 'input'
})

prompts.next({
  name: 'access_token',
  message: 'Enter workspace access token (requires Function Admin permission)',
  type: 'input'
})

// listAllTrackingPlans()

// i.prompt([{
//   message: 'Select language',
//   choices: ['typescript', 'javascript']
// }, {
//   message: 'Select source function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Select destination function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Enter workspace slug',
//   type: 'input'
// }, {
//   message: 'Enter workspace access token (requires Function Admin permission)',
//   type: 'input'
// }]).then(answers => {
//   let [lang, src_name, dest_name, work_slug, access_token] = answers as string[]
//   let cwd = process.cwd()


// })

==> segment-local-functions-0.1.1.tgz: package/src/cli/init.ts <==
#!/usr/bin/env node

import * as i from 'inquirer'
import * as fs from 'fs-extra'
import * as path from 'path'
import { listAllTrackingPlans } from 'segment-typescript-api/cjs/config_api'
import * as Rx from 'rxjs'
import { generate_protocol_definitions } from '../generate_protocol'
import {exec} from 'child_process'

const TEMPLATE_DIR = path.join(__dirname, '..', '..', 'template')
const SAM_DIR = path.join(TEMPLATE_DIR, 'sam')
const JS_DIR = path.join(TEMPLATE_DIR, 'js')
const JS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'js-project')
const TS_DIR = path.join(TEMPLATE_DIR, 'ts')
const TS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'ts-project')

let cwd = process.cwd()

let answers: {
  language: 'typescript' | 'javascript'
  fn_name: string
  fn_type: 'source' | 'destination'
  work_slug: string
  access_token: string
  tp_name: string
} = {
  fn_name: '',
  fn_type: 'source',
  language: 'javascript',
  work_slug: '',
  access_token: '',
  tp_name: ''
}

var prompts = new Rx.Subject();
i.prompt(prompts as any).ui.process.subscribe(answer => {
  let _answer = answer as any
  (answers as any)[_answer.name as keyof typeof answers] = _answer.answer
  // answers = { ...answers, ...answer as Record<keyof typeof answers, any> }

  if (answers.tp_name) {
    prompts.complete()
  } else if (answers.work_slug && answers.access_token) {
    listAllTrackingPlans(answers.access_token, answers.work_slug).then(tps => {
      prompts.next({
        name: 'tp_name',
        message: 'Select tracking plan',
        type: 'list',
        choices: ['none', ...tps.tracking_plans.map(tps => tps.display_name)]
      })
    }).catch(e => {
      console.error('Error fetching tracking plans')
      console.error(e)
      prompts.complete()
    })
  }
}, error => {
  console.error(error)
}, () => {
  let p1 = fs.copyFile(path.join(SAM_DIR, 'package.json'), path.join(cwd, 'package.json')).then(() => {
    exec('npm i', (error, stdout) => {
      if (error) {
        console.error(error)
      }
    })
  })
  let p2 = fs.copyFile(path.join(SAM_DIR, 'template.yaml'), path.join(cwd, 'template.yaml'))
  let p3: Promise<void>
  let p4: Promise<void>
  let p5: Promise<void>
  let p6: Promise<void>

  if (answers.fn_type == 'source') {
    p5 = fs.copy(path.join(JS_DIR, 'tests', 'src-payload.json'), path.join(cwd, 'src', 'tests', 'src-payload.json'))
    if (answers.language == 'javascript') {
      p3 = fs.copyFile(path.join(JS_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'))
      p4 = fs.copy(path.join(JS_DIR, 'source.js'), path.join(cwd, 'src', 'function.js'))
      p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-src-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'))
    } else {
      p3 = fs.copyFile(path.join(TS_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'))
      p4 = fs.copy(path.join(TS_DIR, 'source.ts'), path.join(cwd, 'src', 'function.ts'))
      p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-src-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'))
    }
  } else {
    p3 = fs.copyFile(path.join(SAM_DIR, 'sam-dest.js'), path.join(cwd, 'sam.js'))
    p5 = fs.copy(path.join(JS_DIR, 'tests', 'dest-payload.json'), path.join(cwd, 'src', 'tests', 'dest-payload.json'))
    if (answers.language == 'javascript') {
      p3 = fs.copyFile(path.join(JS_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'))
      p4 = fs.copy(path.join(JS_DIR, 'destination.js'), path.join(cwd, 'src', 'function.js'))
      p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-dest-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'))
    } else {
      p3 = fs.copyFile(path.join(TS_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'))
      p4 = fs.copy(path.join(TS_DIR, 'destination.ts'), path.join(cwd, 'src', 'function.ts'))
      p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-dest-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'))
    }
  }

  let p7: Promise<void>
  if (answers.language == 'javascript') {
    p7 = fs.copy(JS_PROJECT_DIR, cwd)
  } else {
    p7 = fs.copy(TS_PROJECT_DIR, cwd)
  }

  if (answers.tp_name !== 'none') {
    generate_protocol_definitions({
      access_token: answers.access_token,
      tp_name: answers.tp_name,
      work_slug: answers.work_slug,
      out_file: path.join(cwd, 'src', 'protocol.d.ts')
    })
  }
});

prompts.next({
  name: 'language',
  message: 'Select language',
  type: 'list',
  choices: ['javascript', 'typescript']
})

prompts.next({
  name: 'fn_type',
  message: 'Select function type',
  type: 'list',
  choices: ['source', 'destination']
})

prompts.next({
  name: 'fn_name',
  message: 'Select function name',
  type: 'input'
})

prompts.next({
  name: 'work_slug',
  message: 'Enter workspace slug',
  type: 'input'
})

prompts.next({
  name: 'access_token',
  message: 'Enter workspace access token (requires Function Admin permission)',
  type: 'input'
})

// listAllTrackingPlans()

// i.prompt([{
//   message: 'Select language',
//   choices: ['typescript', 'javascript']
// }, {
//   message: 'Select source function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Select destination function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Enter workspace slug',
//   type: 'input'
// }, {
//   message: 'Enter workspace access token (requires Function Admin permission)',
//   type: 'input'
// }]).then(answers => {
//   let [lang, src_name, dest_name, work_slug, access_token] = answers as string[]
//   let cwd = process.cwd()


// })
